@@ -9,7 +9,7 @@ use File::Basename 'dirname';
 use File::Temp 'tempfile';
 sub prompt { ExtUtils::MakeMaker::prompt(@_) }
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 sub new {
     my $class = shift;
@@ -37,6 +37,7 @@ sub parse_options {
 sub run {
     my $self = shift;
     for my $file (@{ $self->{file} }) {
+        next unless -f $file && !-l $file;
         next unless $self->is_perl_shebang( $file );
         unless ($self->{force}) {
             my $anser = prompt "change shebang line of $file? (y/N)", "N";
@@ -54,20 +55,27 @@ sub is_perl_shebang {
     return $first =~ /^#!([^\n]*)perl/ ? 1 : 0;
 }
 
+my $remove = do {
+    my $s = qr/[ \t]*/;
+    my $w = qr/[^\n]*/;
+    my $running_under_some_shell = qr/\n*
+        $s eval $s ['"] exec $w \n
+            $s if $s (?:0|\$running_under_some_shell) $w \n
+    /xsm;
+    my $shebang = qr/\n*
+        \#! $w \n
+    /xsm;
+    qr/\A(?:$running_under_some_shell|$shebang)+/;
+};
+
 sub change_shebang {
     my ($self, $file) = @_;
     my $content = do {
         open my $fh, "<:raw", $file or die "open $file: $!\n";
-        my (undef, @content) = <$fh>; # ignore original shebang
-        join "", @content;
+        local $/; <$fh>;
     };
 
-    # remove "not running under some shell" stuff
-    $content =~ s{\A
-        \s*
-        eval \s* 'exec [^\n]* \n
-        \s* if \s* 0 [^\n]* \n
-    }{}xsm;
+    $content =~ s/$remove//;
 
     my $mode = (stat $file)[2];
 
@@ -99,7 +107,7 @@ App::ChangeShebang - change shebang lines for relocatable perl
 
     > head -3 /path/to/bin/script.pl
     #!/bin/sh
-    exec "$(dirname "$0")"/perl -x -- "$0" "$@"
+    exec "$(dirname "$0")"/perl -x "$0" "$@"
     #!perl
 
 =head1 DESCRIPTION
@@ -111,7 +119,7 @@ L<change-shebang> changes shebang lines from
 to
 
     #!/bin/sh
-    exec "$(dirname "$0")"/perl -x -- "$0" "$@"
+    exec "$(dirname "$0")"/perl -x "$0" "$@"
     #!perl
 
 Why do we need this?
@@ -127,7 +135,7 @@ we cannot execute scripts. Oops!
 A solution of that problem is to replace shebang lines by
 
     #!/bin/sh
-    exec "$(dirname "$0")"/perl -x -- "$0" "$@"
+    exec "$(dirname "$0")"/perl -x "$0" "$@"
     #!perl
 
 which means that scripts will be executed by the perl located in the same directory.
